@@ -1,5 +1,5 @@
 /*
- * $Id: com-misc.c,v 1.8.2.1 2002/01/28 01:53:07 mt Exp $
+ * $Id: com-misc.c,v 1.9 2002/05/02 12:59:00 mt Exp $
  *
  * Common miscellaneous functions
  *
@@ -30,7 +30,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: com-misc.c,v 1.8.2.1 2002/01/28 01:53:07 mt Exp $";
+static char rcsid[] = "$Id: com-misc.c,v 1.9 2002/05/02 12:59:00 mt Exp $";
 #endif
 
 #include <config.h>
@@ -784,6 +784,9 @@ int misc_rand (int lrng, int urng)
 
 /* ------------------------------------------------------------
  * $Log: com-misc.c,v $
+ * Revision 1.9  2002/05/02 12:59:00  mt
+ * merged with v1.8.2.2
+ *
  * Revision 1.8.2.1  2002/01/28 01:53:07  mt
  * implemented misc_strnequ misc_strncaseequ wrappers
  *
