@@ -1,5 +1,5 @@
 /*
- * $Id: com-socket.c,v 1.6.2.2 2002/04/04 14:44:30 mt Exp $
+ * $Id: com-socket.c,v 1.7 2002/05/02 13:01:08 mt Exp $
  *
  * Common functions for TCP/IP sockets
  *
@@ -30,7 +30,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: com-socket.c,v 1.6.2.2 2002/04/04 14:44:30 mt Exp $";
+static char rcsid[] = "$Id: com-socket.c,v 1.7 2002/05/02 13:01:08 mt Exp $";
 #endif
 
 #include <config.h>
@@ -1326,6 +1326,8 @@ char *socket_msgline(char *fmt)
 			case '%':
 				tmp[0] = '%';
 				break;
+			default:
+			break;
 		}
 		tmp[sizeof(tmp)-1] = '\0'; /* paranoia :-) */
 		j = strlen(tmp);
@@ -2185,6 +2187,9 @@ int getfqdomainname(char *fqdomain, size_t n)
 
 /* ------------------------------------------------------------
  * $Log: com-socket.c,v $
+ * Revision 1.7  2002/05/02 13:01:08  mt
+ * merged with v1.8.2.2
+ *
  * Revision 1.6.2.2  2002/04/04 14:44:30  mt
  * added waiting and retrying on "no data" but no EOF in I_NREAD
  * added check for buffer len difference on ioctl and recv
