@@ -1,5 +1,5 @@
 /*
- * $Id: ftp-client.c,v 1.8.2.1 2002/04/04 14:26:11 mt Exp $
+ * $Id: ftp-client.c,v 1.9 2002/05/02 13:15:36 mt Exp $
  *
  * Functions for the FTP Proxy client handling
  *
@@ -30,7 +30,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: ftp-client.c,v 1.8.2.1 2002/04/04 14:26:11 mt Exp $";
+static char rcsid[] = "$Id: ftp-client.c,v 1.9 2002/05/02 13:15:36 mt Exp $";
 #endif
 
 #include <config.h>
@@ -487,8 +487,19 @@ void client_run(void)
 	/*
 	** Free allocated memory
 	*/
-	if (ctx.username != NULL)
+	ctx.magic_auth = NULL;
+	if (ctx.userauth != NULL) {
+		misc_free(FL, ctx.userauth);
+		ctx.userauth = NULL;
+	}
+	if (ctx.username != NULL) {
 		misc_free(FL, ctx.username);
+		ctx.username = NULL;
+	}
+	if(ctx.userpass != NULL) {
+		misc_free(FL, ctx.userpass);
+		ctx.userpass = NULL;
+	}
 
 #if defined(COMPILE_DEBUG)
 	debug(1, "}}}}} %s client-exit", misc_getprog());
@@ -691,6 +702,8 @@ static void client_srv_ctrl_read(char *str)
 		*/
 		if (ctx.expect == EXP_CONN)
 			return;
+		if (ctx.expect == EXP_USER && NULL != ctx.userauth)
+			return;
 
 #if defined(COMPILE_DEBUG)
 		debug(2, "'%.4s'... forwarded to %s %d=%s", str,
@@ -725,11 +738,17 @@ static void client_srv_ctrl_read(char *str)
 			*/
 			if (c1 == 2) {
 				socket_printf(ctx.srv_ctrl,
-					"USER %s\r\n", ctx.username);
+				              "USER %s\r\n",
+				              ctx.username);
 				ctx.expect = EXP_USER;
 			} else {
-				socket_printf(ctx.cli_ctrl,
-						"%s\r\n", str);
+				if(NULL != ctx.userauth) {
+					client_respond(530, NULL,
+					               "Login incorrect");
+				} else {
+					socket_printf(ctx.cli_ctrl,
+					              "%s\r\n", str);
+				}
 				ctx.expect = EXP_IDLE;
 				ctx.cli_ctrl->kill = 1;
 			}
@@ -737,8 +756,41 @@ static void client_srv_ctrl_read(char *str)
 
 		case EXP_USER:
 			/*
-			** Only the following are useful:
-			** 230=logged in, 331=need password
+			** Only the following codes are useful:
+			**	230=logged in,
+			**	331=need password,
+			**	332=need password+account
+			*/
+			if(NULL != ctx.userauth) {
+				/*
+				** logged in, NO password needed
+				*/
+				if(c1 == 2 && c2 == 3) {
+					client_respond(230, NULL,
+					"User logged in, proceed.");
+					ctx.expect = EXP_IDLE;
+					break;
+				} else
+				/*
+				** OK, password (+account) needed
+				*/
+				if(c1 == 3 && c2 == 3) {
+					if(ctx.userpass) {
+						socket_printf(ctx.srv_ctrl,
+						              "PASS %s\r\n",
+						              ctx.userpass);
+						misc_free(FL, ctx.userpass);
+						ctx.userpass = NULL;
+					} else {
+						socket_printf(ctx.srv_ctrl,
+						              "PASS \r\n");
+					}
+					ctx.expect = EXP_PTHR;
+					break;
+				}
+			}
+			/*
+			** pass server response through to client
 			*/
 			socket_printf(ctx.cli_ctrl, "%s\r\n", str);
 			if (c1 != 2 && c1 != 3) {
@@ -857,6 +909,9 @@ static void client_srv_passive(char *arg)
 	}
 	addr = (u_int32_t) ((h1 << 24) + (h2 << 16) + (h3 << 8) + h4);
 	port = (u_int16_t) ((p1 <<  8) +  p2);
+	syslog_write(T_DBG, "got SRV-PASV %s:%d for %s:%d",
+	                    socket_addr2str(addr), port,
+	                    ctx.cli_ctrl->peer, ctx.cli_ctrl->port);
 
 	/*
 	** should we bind a rand(port-range) or increment?
@@ -1048,10 +1103,19 @@ void client_reinit(void)
 	/*
 	** Remove the current user and status
 	*/
+	ctx.magic_auth = NULL;
+	if (ctx.userauth != NULL) {
+		misc_free(FL, ctx.userauth);
+		ctx.userauth = NULL;
+	}
 	if (ctx.username != NULL) {
 		misc_free(FL, ctx.username);
 		ctx.username = NULL;
 	}
+	if(ctx.userpass != NULL) {
+		misc_free(FL, ctx.userpass);
+		ctx.userpass = NULL;
+	}
 	ctx.expect = EXP_IDLE;
 }
 
@@ -1090,8 +1154,241 @@ void client_data_reset(int mode)
 }
 
 
+/* ------------------------------------------------------------ **
+**
+**	Function......:	client_srv_open
+**
+**	Parameters....:	(none)
+**
+**	Return........:	(none)
+**
+**	Purpose.......: Open control connection to the server.
+**
+** ------------------------------------------------------------ */
+
+void client_srv_open(void)
+{
+	struct sockaddr_in saddr;
+	u_int16_t          lprt, lowrng, res;
+	int                sock, incr, retry;
+
+	/*
+	** should we bind a rand(port-range) or increment?
+	*/
+	incr = !config_bool(NULL,"SockBindRand", 0);
+
+	/*
+	** mark socket invalid
+	*/
+	sock = -1;
+
+	/*
+	** Forward connection to destination
+	*/
+	retry = MAX_RETRIES;
+	lprt  = ctx.srv_lrng;
+	while(0 <= retry--) {
+		/*
+		** First of all, get a socket
+		*/
+		if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+			syslog_error("Srv-Ctrl: can't create socket for %s",
+			             ctx.cli_ctrl->peer);
+			exit(EXIT_FAILURE);
+		}
+		socket_opts(sock, SK_CONTROL);
+
+		/*
+		** check if we have to take care to a port range
+		*/
+		if( !(INPORT_ANY == ctx.srv_lrng &&
+                      INPORT_ANY == ctx.srv_urng))
+		{
+			u_int32_t ladr = INADDR_ANY;
+
+			/*
+			** bind the socket, taking care of a given port range
+			*/
+			if(incr) {
+				lowrng = lprt;
+#if defined(COMPILE_DEBUG)
+				debug(2, "Srv-Ctrl: "
+				         "about to bind to %s:range(%d-%d)",
+				         socket_addr2str(ladr),
+				         lowrng, ctx.srv_urng);
+#endif
+				res = socket_d_bind(sock, ladr,
+				       lowrng, ctx.srv_urng, incr);
+			} else {
+				lowrng = ctx.srv_lrng;
+#if defined(COMPILE_DEBUG)
+				debug(2, "Srv-Ctrl: "
+				         "about to bind to %s:range(%d-%d)",
+				         socket_addr2str(ladr),
+				         lowrng, ctx.srv_urng);
+#endif
+				res = socket_d_bind(sock, ladr,
+				           lowrng, ctx.srv_urng, incr);
+			}
+
+			if (INPORT_ANY == res) {
+				/* nothing found? */
+				close(sock);
+				syslog_error("Srv-Ctrl: can't bind to"
+				             " %s:%d for %s",
+					     socket_addr2str(ladr),
+					     (int)lprt, ctx.cli_ctrl->peer);
+				exit(EXIT_FAILURE);
+			} else {
+				lprt = res;
+			}
+		} else lprt = INPORT_ANY;
+
+		/*
+		** Okay, now try the actual connect to the server
+		*/
+		memset(&saddr, 0, sizeof(saddr));
+		saddr.sin_addr.s_addr = htonl(ctx.srv_addr);
+		saddr.sin_family      = AF_INET;
+		saddr.sin_port        = htons(ctx.srv_port);
+
+		if (connect(sock, (struct sockaddr *)&saddr,
+		            sizeof(saddr)) < 0)
+		{
+#if defined(COMPILE_DEBUG)
+				debug(2, "Srv-Ctrl: connect failed with '%s'",
+				      strerror(errno));
+#endif
+			close(sock);
+			sock = -1;
+			/* check if is makes sense to retry?
+			** perhaps we only need an other
+			** local port (EADDRNOTAVAIL) for
+			** this destination?
+			*/
+			if( !(EINTR == errno ||
+			      EAGAIN == errno ||
+			      EADDRINUSE == errno ||
+			      EADDRNOTAVAIL == errno))
+			{
+				/*
+				** an other (real) error ocurred
+				*/
+				syslog_error("Srv-Ctrl: "
+				             "can't connect %s:%d for %s",
+				             socket_addr2str(ctx.srv_addr),
+				             (int) ctx.srv_port,
+				             ctx.cli_ctrl->peer);
+				exit(EXIT_FAILURE);
+			}
+			if(incr && INPORT_ANY != lprt) {
+				/* increment lower range if we use
+				** increment mode and have a range
+				*/
+				if(lprt < ctx.srv_urng) {
+					lprt++;
+				} else {
+				/*
+				** no more ports in range we can try
+				*/
+				syslog_error("Srv-Ctrl: "
+				             "can't connect %s:%d for %s",
+				             socket_addr2str(ctx.srv_addr),
+				             (int) ctx.srv_port,
+				             ctx.cli_ctrl->peer);
+				exit(EXIT_FAILURE);
+				}
+			}
+		} else break;
+	}
+
+	/*
+	** check if we have a valid, connected socket
+	*/
+	if(-1 == sock) {
+		syslog_error("Srv-Ctrl: can't connect %s:%d for %s",
+		             socket_addr2str(ctx.srv_addr),
+		             (int) ctx.srv_port,
+		             ctx.cli_ctrl->peer);
+		exit(EXIT_FAILURE);
+	}
+
+	if ((ctx.srv_ctrl = socket_init(sock)) == NULL)
+		misc_die(FL, "cmds_user: ?srv_ctrl?");
+	ctx.srv_ctrl->ctyp = "Srv-Ctrl";
+
+#if defined(COMPILE_DEBUG)
+		debug(2, "Srv-Ctrl is %s:%d",
+			ctx.srv_ctrl->peer, (int) ctx.srv_port);
+#endif
+
+	ctx.expect = EXP_CONN;		/* Expect Welcome	*/
+}
+
+
+/* ------------------------------------------------------------ **
+**
+**	Function......:	client_setup
+**
+**	Parameters....:	pwd	client / user auth password
+**
+**	Return........:	0 on success
+**
+**	Purpose.......: setup user-profile and preform auth
+**	                if configured...
+**
+** ------------------------------------------------------------ */
+
+int client_setup(char *pwd)
+{
+	char *type;
+
+	/*
+	** Setup defaults for the client's DTP process;
+	** user specific setup is done in ldap_setup_user
+	*/
+	ctx.cli_mode = MOD_ACT_FTP;
+	ctx.cli_addr = ctx.cli_ctrl->addr;
+	ctx.cli_port = ctx.cli_ctrl->port;
+	ctx.srv_addr = INADDR_ANY;
+	ctx.srv_port = INPORT_ANY;
+
+	/*
+	** check if we have to authenticate user
+	*/
+	if(NULL == (type = config_str(NULL, "UserAuthType", NULL))) {
+		/*
+		** setup use only
+		*/
+		return ldap_setup_user(&ctx, NULL);
+	}
+
+	/*
+	** authenticate + setup
+	*/
+/*
+	if( !strcasecmp(type, "pam")) {
+		if(0 == pam_auth_user(&ctx, pwd)) {
+			return ldap_setup_user(&ctx, NULL);
+		}
+	} else
+*/
+	if( !strcasecmp(type, "ldap")) {
+		if(config_str(NULL, "LDAPServer", NULL) == NULL)
+			misc_die(FL, "client_setup: ?LDAPServer?");
+		return ldap_setup_user(&ctx, pwd);
+	}
+
+	misc_die(FL, "client_setup: ?UserAuthType?");
+	return -1; /* never reached */
+}
+
+
 /* ------------------------------------------------------------
  * $Log: ftp-client.c,v $
+ * Revision 1.9  2002/05/02 13:15:36  mt
+ * implemented simple (ldap based) user auth
+ *
  * Revision 1.8.2.1  2002/04/04 14:26:11  mt
  * added failed/completed transfer log message status
  *
