@@ -1,5 +1,5 @@
 /*
- * $Id: ftp-client.h,v 1.5 2002/01/14 19:35:44 mt Exp $
+ * $Id: ftp-client.h,v 1.6 2002/05/02 13:15:36 mt Exp $
  *
  * Header for the FTP Proxy client handling
  *
@@ -90,7 +90,10 @@ typedef struct {
 	HLS *srv_ctrl;		/* Control path to the server	*/
 	HLS *srv_data;		/* Data path to the server	*/
 
-	char *username;		/* Our client's user name	*/
+	char *username;		/* Client's ftp-username	*/
+	char *userpass;		/* Client's ftp-password	*/
+	char *userauth;		/* Client's user auth name	*/
+	char *magic_auth;	/* userauth magic mode-str	*/
 
 	u_int32_t magic_addr;	/* The "real" destination ...	*/
 	u_int16_t magic_port;	/* ... and corresponding port	*/
@@ -133,16 +136,22 @@ typedef struct {
 /* ------------------------------------------------------------ */
 
 void client_run    (void);
-void client_respond(int code, char *file, char *fmt, ...);
 void client_reinit (void);
+void client_respond(int code, char *file, char *fmt, ...);
 void client_data_reset(int mode);
 
+int  client_setup(char *pwd);
+void client_srv_open(void);
+
 /* ------------------------------------------------------------ */
 
 #endif /* defined(_FTP_CLIENT_H_) */
 
 /* ------------------------------------------------------------
  * $Log: ftp-client.h,v $
+ * Revision 1.6  2002/05/02 13:15:36  mt
+ * implemented simple (ldap based) user auth
+ *
  * Revision 1.5  2002/01/14 19:35:44  mt
  * implemented workarround for Netscape (4.x) directory symlink handling
  * implemented a MaxRecvBufSize option limiting max recv buffer size
