@@ -1,5 +1,5 @@
 /*
- * $Id: ftp-cmds.c,v 1.9.2.1 2002/04/04 14:23:32 mt Exp $
+ * $Id: ftp-cmds.c,v 1.10 2002/05/02 13:16:35 mt Exp $
  *
  * FTP Proxy command handling
  *
@@ -30,7 +30,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: ftp-cmds.c,v 1.9.2.1 2002/04/04 14:23:32 mt Exp $";
+static char rcsid[] = "$Id: ftp-cmds.c,v 1.10 2002/05/02 13:16:35 mt Exp $";
 #endif
 
 #include <config.h>
@@ -76,7 +76,6 @@ static char rcsid[] = "$Id: ftp-cmds.c,v 1.9.2.1 2002/04/04 14:23:32 mt Exp $";
 #include "com-syslog.h"
 #include "ftp-client.h"
 #include "ftp-cmds.h"
-#include "ftp-ldap.h"
 
 
 /* ------------------------------------------------------------ */
@@ -346,12 +345,9 @@ static void cmds_pthr(CONTEXT *ctx, char *arg)
 
 static void cmds_user(CONTEXT *ctx, char *arg)
 {
-	struct sockaddr_in saddr;
-	int                sock, len;
-	int                incr, retry;
-	u_int16_t          lprt, lowrng, bind_res;
-	char               *p, *q;
 	CMD                *cmd;
+	char               *p, *q;
+	int                need_auth;
 
 	if (ctx == NULL)		/* Basic sanity check	*/
 		misc_die(FL, "cmds_user: ?ctx?");
@@ -395,6 +391,7 @@ static void cmds_user(CONTEXT *ctx, char *arg)
 	}
 #endif
 
+
 	/*
 	** Check for permission and existence of "transparent proxy"
 	** magic destination address and port from the client socket
@@ -404,8 +401,6 @@ static void cmds_user(CONTEXT *ctx, char *arg)
 	*/
 	ctx->magic_addr = INADDR_ANY;
 	ctx->magic_port = INPORT_ANY;
-	len = sizeof(saddr);
-	memset(&saddr, 0, len);
 
 	if(config_bool(NULL, "AllowTransProxy", 0)) {
 		u_int32_t addr = INADDR_ANY;
@@ -436,13 +431,94 @@ static void cmds_user(CONTEXT *ctx, char *arg)
 			break;
 			default:
 			syslog_write(T_DBG,
-			  "requested transparent destination %s is local",
-			  socket_addr2str(ntohl(addr)));
+				"requested transparent destination %s is local",
+				socket_addr2str(ntohl(addr)));
 			break;
 			}
 		} else {
 			syslog_write(T_DBG,
-			"no transparent proxy destination found");
+				"no transparent proxy destination found");
+		}
+	}
+
+	/*
+	** Check if we should use the auth-user mode
+	*/
+	need_auth = 0;
+	if(NULL != config_str(NULL, "UserAuthType", NULL)) {
+		need_auth = 1;
+		ctx->magic_auth = config_str(NULL, "UserAuthMagic", NULL);
+	}
+
+	/*
+	** Check if we have to parse 'auth' user name...
+	*/
+	if(need_auth && (NULL != ctx->magic_auth)) {
+#if defined(COMPILE_DEBUG)
+		debug(2, "parsing '%s' using auth-magic '%s'",
+			arg, ctx->magic_auth);
+#endif
+		/*
+		** Check how to parse for a client auth-name...
+		** type1: auth@user[@host:port] or
+		** type2: user@auth[@host:port] (NcFTP type5)
+		*/
+		if(sizeof("auth") == strlen(ctx->magic_auth) &&
+                   !strncasecmp(ctx->magic_auth, "auth", sizeof("auth")-1))
+		{
+			if((q = strchr(arg, ctx->magic_auth[sizeof("auth")-1])))
+			{
+				*q++ = '\0';
+				if( strlen(arg)) {
+					ctx->userauth = misc_strdup(FL, arg);
+				}
+				arg  = q;
+			}
+		} else
+		if((q = strchr(arg, ctx->magic_auth[0]))) {
+			/*
+			** @auth may be followed by @host:port
+			*/
+			if(config_bool(NULL, "AllowMagicUser", 0) &&
+			   (p = config_str(NULL, "UseMagicChar", "@")) &&
+			   (p = strchr(q+1, p[0])))
+			{
+				if(p>q+1) {
+					ctx->userauth = misc_alloc(FL, p-q);
+					misc_strncpy(ctx->userauth, q+1, p-q);
+				}
+				while(*p) *q++=*p++;
+				*q = '\0';
+			} else {
+				*q++ = '\0';
+				if( strlen(q)) {
+					ctx->userauth = misc_strdup(FL, q);
+				}
+			}
+		}
+
+		syslog_write(T_DBG, "parsed (magic)user='%s', auth='%s'",
+		                    NIL(arg), NIL(ctx->userauth));
+
+		/*
+		** Check if non-empty auth is given
+		*/
+		if(NULL == ctx->userauth) {
+			client_respond(501, NULL, "Permission denied");
+			syslog_write(U_WRN, "'USER' without auth from %s",
+			                    ctx->cli_ctrl->peer);
+			return;
+		}
+
+		/*
+		** Check for the user name - again...
+		*/
+		if (arg == NULL || *arg == '\0') {
+			client_respond(501, NULL, "Missing user name");
+			syslog_write(U_WRN, "'USER' without name from %s",
+			                    ctx->cli_ctrl->peer);
+			client_reinit();
+			return;
 		}
 	}
 
@@ -459,8 +535,9 @@ static void cmds_user(CONTEXT *ctx, char *arg)
 		if ((q = strchr(p, ':')) != NULL) {
 			*q++ = '\0';
 			ctx->magic_port = socket_str2port(q, IPPORT_FTP);
-		} else
+		} else {
 			ctx->magic_port = IPPORT_FTP;
+		}
 		ctx->magic_addr = socket_str2addr(p, INADDR_ANY);
 		if (ctx->magic_addr == INADDR_ANY ||
 		    ctx->magic_port == INPORT_ANY) {
@@ -469,6 +546,18 @@ static void cmds_user(CONTEXT *ctx, char *arg)
 			syslog_write(U_WRN,
 				"bad 'USER@' dest '%s:%s' from %s",
 				NIL(p), NIL(q), ctx->cli_ctrl->peer);
+			client_reinit();
+			return;
+		}
+
+		/*
+		** Check for the user name - again...
+		*/
+		if (arg == NULL || *arg == '\0') {
+			client_respond(501, NULL, "Missing user name");
+			syslog_write(U_WRN, "'USER' without name from %s",
+			                    ctx->cli_ctrl->peer);
+			client_reinit();
 			return;
 		}
 	}
@@ -484,164 +573,49 @@ static void cmds_user(CONTEXT *ctx, char *arg)
 	if(config_str(NULL, "DestinationAddress", NULL) == NULL) {
 		syslog_write(U_ERR, "unknown destination address");
 		client_respond(501, NULL,"Unknown destination address");
+		client_reinit();
 		return;
 	} else {
 		syslog_write(U_INF, "'USER %s' from %s",
 				arg, ctx->cli_ctrl->peer);
 	}
+
+	/*
+	** remember ftp user name
+	*/
 	ctx->username = misc_strdup(FL, arg);
-	ldap_setup_user(ctx);
 
 	/*
-	** should we bind a rand(port-range) or increment?
+	** copy normal ftp user name into userauth
+	** if we need any auth but have no userauth
 	*/
-	incr = !config_bool(NULL,"SockBindRand", 0);
+	if(need_auth && (NULL == ctx->userauth)) {
+		ctx->userauth = misc_strdup(FL, arg);
+	}
 
-	/*
-	** mark socket invalid
-	*/
-	sock = -1;
-
-	/*
-	** Forward connection to destination
-	*/
-	retry = MAX_RETRIES;
-	lprt  = ctx->srv_lrng;
-	while(0 <= retry--) {
+	if(NULL != ctx->userauth) {
 		/*
-		** First of all, get a socket
+		** anable PASS command only...
 		*/
-		if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-			syslog_error("Srv-Ctrl: can't create socket for %s",
-					ctx->cli_ctrl->peer);
-			exit(EXIT_FAILURE);
+		cmds_set_allow("PASS");
+
+		/*
+		** hmm... a USER name is there, but we need
+		** PASS+auth as well, because it may be needed
+		** for auth itself and for profile reading...
+		*/
+		client_respond(331, NULL,
+		               "User name okay, need password.");
+	} else {
+		/*
+		** read user's profile, connect the server
+		*/
+		if(0 == client_setup(NULL)) {
+			client_srv_open();
+		} else {
+			client_reinit();
 		}
-		socket_opts(sock, SK_CONTROL);
-
-		/*
-		** check if we have to take care to a port range
-		*/
-		if( !(INPORT_ANY == ctx->srv_lrng &&
-                      INPORT_ANY == ctx->srv_urng))
-		{
-			u_int32_t ladr = INADDR_ANY;
-
-			/*
-			** bind the socket, taking care of a given port range
-			*/
-			if(incr) {
-				lowrng = lprt;
-#if defined(COMPILE_DEBUG)
-				debug(2, "Srv-Ctrl: "
-				         "about to bind to %s:range(%d-%d)",
-				         socket_addr2str(ladr),
-				         lowrng, ctx->srv_urng);
-#endif
-				bind_res = socket_d_bind(sock, ladr,
-				       lowrng, ctx->srv_urng, incr);
-			} else {
-				lowrng = ctx->srv_lrng;
-#if defined(COMPILE_DEBUG)
-				debug(2, "Srv-Ctrl: "
-				         "about to bind to %s:range(%d-%d)",
-				         socket_addr2str(ladr),
-				         lowrng, ctx->srv_urng);
-#endif
-				bind_res = socket_d_bind(sock, ladr,
-				           lowrng, ctx->srv_urng, incr);
-			}
-			if (INPORT_ANY == bind_res) {
-				/* nothing found? */
-				close(sock);
-				syslog_error("Srv-Ctrl: can't bind to"
-				             " %s:%d for %s",
-					     socket_addr2str(ladr),
-					     (int)lprt, ctx->cli_ctrl->peer);
-				exit(EXIT_FAILURE);
-			} else {
-				lprt = bind_res;
-			}
-		} else lprt = INPORT_ANY;
-
-		/*
-		** Okay, now try the actual connect to the server
-		*/
-		memset(&saddr, 0, sizeof(saddr));
-		saddr.sin_addr.s_addr = htonl(ctx->srv_addr);
-		saddr.sin_family      = AF_INET;
-		saddr.sin_port        = htons(ctx->srv_port);
-
-		if (connect(sock, (struct sockaddr *)&saddr,
-		            sizeof(saddr)) < 0)
-		{
-#if defined(COMPILE_DEBUG)
-				debug(2, "Srv-Ctrl: connect failed with '%s'",
-				      strerror(errno));
-#endif
-			close(sock);
-			sock = -1;
-			/* check if is makes sense to retry?
-			** perhaps we only need an other
-			** local port (EADDRNOTAVAIL) for
-			** this destination?
-			*/
-			if( !(EINTR == errno ||
-			      EAGAIN == errno ||
-			      EADDRINUSE == errno ||
-			      EADDRNOTAVAIL == errno))
-			{
-				/*
-				** an other (real) error ocurred
-				*/
-				syslog_error("Srv-Ctrl: "
-				             "can't connect %s:%d for %s",
-				             socket_addr2str(ctx->srv_addr),
-				             (int) ctx->srv_port,
-				             ctx->cli_ctrl->peer);
-				exit(EXIT_FAILURE);
-			}
-			if(incr && INPORT_ANY != lprt) {
-				/* increment lower range if we use
-				** increment mode and have a range
-				*/
-				if(lprt < ctx->srv_urng) {
-					lprt++;
-				} else {
-				/*
-				** no more ports in range we can try
-				*/
-				syslog_error("Srv-Ctrl: "
-				             "can't connect %s:%d for %s",
-				             socket_addr2str(ctx->srv_addr),
-				             (int) ctx->srv_port,
-				             ctx->cli_ctrl->peer);
-				exit(EXIT_FAILURE);
-				}
-			}
-		} else break;
 	}
-
-	/*
-	** check if we have a valid, connected socket
-	*/
-	if(-1 == sock) {
-		syslog_error("Srv-Ctrl: can't connect %s:%d for %s",
-		             socket_addr2str(ctx->srv_addr),
-		             (int) ctx->srv_port,
-		             ctx->cli_ctrl->peer);
-		exit(EXIT_FAILURE);
-	}
-
-	if ((ctx->srv_ctrl = socket_init(sock)) == NULL)
-		misc_die(FL, "cmds_user: ?srv_ctrl?");
-	ctx->srv_ctrl->ctyp = "Srv-Ctrl";
-
-#if defined(COMPILE_DEBUG)
-		debug(2, "Srv-Ctrl is %s:%d",
-			ctx->srv_ctrl->peer, (int) ctx->srv_port);
-#endif
-
-	ctx->expect = EXP_CONN;		/* Expect Welcome	*/
 }
 
 
@@ -660,30 +634,100 @@ static void cmds_user(CONTEXT *ctx, char *arg)
 
 static void cmds_pass(CONTEXT *ctx, char *arg)
 {
+	char *pass, *q;
+
 	if (ctx == NULL)		/* Basic sanity check	*/
 		misc_die(FL, "cmds_pass: ?ctx?");
 
-	if (ctx->srv_ctrl == NULL) {
-		client_respond(530, NULL, "Not logged in");
-		syslog_write(U_WRN, "'PASS' without login from %s",
-						ctx->cli_ctrl->peer);
-		return;
+	/*
+	** inform auditor...
+	*/
+	syslog_write(U_INF, "'PASS XXXX' from %s", ctx->cli_ctrl->peer);
+
+	/*
+	** Check if have to parse auth-pass... may be NULL
+	*/
+	pass = 0;
+	if(NULL != ctx->magic_auth) {
+		/*
+		** Check how to parse for a client auth pass...
+		** type1: auth@pass or type2: pass@auth (NcFTP type5)
+		*/
+		if(NULL != arg && sizeof("auth") == strlen(ctx->magic_auth) &&
+		   !strncasecmp(ctx->magic_auth, "auth", sizeof("auth")-1))
+		{
+			if((q = strchr(arg, ctx->magic_auth[sizeof("auth")-1])))
+			{
+				*q++ = '\0';
+				if(strlen(arg)) {
+					pass = arg;
+				}
+				if( strlen(q)) {
+					ctx->userpass = misc_strdup(FL, q);
+				}
+			}
+		} else {
+			if((NULL != arg) &&
+			   (q = strrchr(arg, ctx->magic_auth[0])))
+			{
+				*q++ = '\0';
+				if( strlen(q)) {
+					pass = q;
+				}
+				if(strlen(arg)) {
+					ctx->userpass = misc_strdup(FL, arg);
+				}
+			}
+		}
+	} else {
+		/* we may use normal ftp pwd */
+		pass = arg;
 	}
 
 	/*
-	** If no password is supplied, send none either
+	** check if we are in auth mode
 	*/
-	if (arg == NULL)
-		arg = "";
+	if(NULL != ctx->userauth) {
+		/*
+		** OK, we have all data to auth user, read
+		** his proxy-profile and connect to server
+		*/
+		if(0 == client_setup(pass)) {
+			client_srv_open();
+		} else {
+			client_respond(530, NULL, "Not logged in");
+			client_reinit();
+		}
+		return;
 
-	/*
-	** Send to server, but do not display
-	*/
-	socket_printf(ctx->srv_ctrl, "PASS %.1024s\r\n", arg);
-	syslog_write(U_INF,
-		"'PASS XXXX' from %s", ctx->cli_ctrl->peer);
+	} else {
 
-	ctx->expect = EXP_PTHR;		/* Expect Response	*/
+		/*
+		** paranoia check...
+		*/
+		if (ctx->srv_ctrl == NULL) {
+			client_respond(530, NULL, "Not logged in");
+			syslog_write(U_WRN, "'PASS' without login from %s",
+			             ctx->cli_ctrl->peer);
+			return;
+		}
+
+		/*
+		** If no password is supplied, send none either
+		*/
+		if (arg == NULL)
+			arg = "";
+
+		/*
+		** Send to server, but do not display
+		*/
+		socket_printf(ctx->srv_ctrl, "PASS %.1024s\r\n", arg);
+		syslog_write(U_INF, "'PASS XXXX' from %s",
+		             ctx->cli_ctrl->peer);
+
+		/* Expect Response */
+		ctx->expect = EXP_PTHR;
+	}
 }
 
 
@@ -1363,6 +1407,9 @@ char *cmds_reg_exec(void *regex, char *str)
 
 /* ------------------------------------------------------------
  * $Log: ftp-cmds.c,v $
+ * Revision 1.10  2002/05/02 13:16:35  mt
+ * implemented simple (ldap based) user auth
+ *
  * Revision 1.9.2.1  2002/04/04 14:23:32  mt
  * improved transparent proxy log messages
  *
