@@ -1,5 +1,5 @@
 /*
- * $Id: ftp-ldap.c,v 1.6 2002/01/14 19:26:38 mt Exp $
+ * $Id: ftp-ldap.c,v 1.7 2002/05/02 13:17:12 mt Exp $
  *
  * FTP Proxy LDAP interface handling
  *
@@ -30,11 +30,13 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: ftp-ldap.c,v 1.6 2002/01/14 19:26:38 mt Exp $";
+static char rcsid[] = "$Id: ftp-ldap.c,v 1.7 2002/05/02 13:17:12 mt Exp $";
 #endif
 
 #include <config.h>
 
+#define _GNU_SOURCE		/* needed for crypt in Linux... */
+
 #if defined(STDC_HEADERS)
 #  include <stdio.h>
 #  include <string.h>
@@ -70,7 +72,7 @@ static char rcsid[] = "$Id: ftp-ldap.c,v 1.6 2002/01/14 19:26:38 mt Exp $";
 #  endif
 #  include <ldap.h>
 #  if !defined(LDAP_PORT)
-#    define LDAP_PORT	389
+#    define LDAP_PORT		389
 #  endif
 #endif
 
@@ -83,7 +85,6 @@ static char rcsid[] = "$Id: ftp-ldap.c,v 1.6 2002/01/14 19:26:38 mt Exp $";
 #include "ftp-cmds.h"
 #include "ftp-ldap.h"
 
-
 /* ------------------------------------------------------------ */
 
 #if defined(HAVE_LIBLDAP)
@@ -121,16 +122,19 @@ static char rcsid[] = "$Id: ftp-ldap.c,v 1.6 2002/01/14 19:26:38 mt Exp $";
 /* ------------------------------------------------------------ */
 
 #if defined(HAVE_LIBLDAP)
-static int   ldap_fetch (CONTEXT *ctx, char *srv);
+static int   ldap_fetch(CONTEXT *ctx, char *srv, char *who, char *pwd);
+static int   ldap_auth(LDAP *ld, LDAPMessage *e, char *who, char *pwd);
 static char *ldap_attrib(LDAP *ld, LDAPMessage *e, char *attr, char *dflt);
+static int   ldap_exists(LDAP *ld, LDAPMessage *e, char *attr,
+                                                   char *vstr, int cs);
 #endif
 
-
 /* ------------------------------------------------------------ **
 **
 **	Function......:	ldap_setup_user
 **
 **	Parameters....:	ctx		Pointer to user context
+**			pwd		Pointer to user auth pwd
 **
 **	Return........:	(none)
 **
@@ -140,51 +144,50 @@ static char *ldap_attrib(LDAP *ld, LDAPMessage *e, char *attr, char *dflt);
 **
 ** ------------------------------------------------------------ */
 
-void ldap_setup_user(CONTEXT *ctx)
+int  ldap_setup_user(CONTEXT *ctx, char *pwd)
 {
-	char *p;
+	char      *who, *p;
 	u_int16_t l, u;
 
 	if (ctx == NULL)		/* Basic sanity check	*/
 		misc_die(FL, "ldap_setup_user: ?ctx?");
 
 	/*
-	** Setup defaults for the client's DTP process
+	** remember the proper name we should use...
 	*/
-	ctx->cli_mode = MOD_ACT_FTP;
-	ctx->cli_addr = ctx->cli_ctrl->addr;
-	ctx->cli_port = ctx->cli_ctrl->port;
+	if(NULL != ctx->userauth) {
+		who = ctx->userauth;
+	} else {
+		who = ctx->username;
+	}
 
-	ctx->srv_addr = (u_int32_t) 0;	/* Let's set a marker	*/
+	/*
+	** dont allow empty or invalid names...
+	*/
+	pwd=pwd;
+	if( !(NULL != who && *who && !strchr(who, '*'))) {
+		syslog_error("invalid user name or with asterisk");
+		return -1;
+	}
 
 #if defined(HAVE_LIBLDAP)
 	/*
 	** If an LDAP server is configured, insist on using it
 	*/
 	if ((p = config_str(NULL, "LDAPServer", NULL)) != NULL) {
-		int lderr;
-		if ((lderr = ldap_fetch(ctx, p)) != LDAP_SUCCESS) {
-			errno = 0;
-			syslog_error("can't read LDAP data "
-					"for %s: %.512s",
-					ctx->cli_ctrl->peer,
-					ldap_err2string(lderr));
-			exit(EXIT_FAILURE);
-		}
-		if (ctx->srv_addr != (u_int32_t) 0) {
-			syslog_write(U_INF,
-				"reading data for '%s' from LDAP",
-				ctx->username);
-			return;		/* LDAP has delivered	*/
-		}
+		int rc = ldap_fetch(ctx, p, who, pwd);
+		/*
+		** check if we have to read profile from config...
+		*/
+		syslog_write(T_DBG, "ldap_fetch returned %d", rc);
+		if(0 != rc) return rc;
 	}
 #endif
 
 	/*
 	** Inform the auditor that we are using the config file
 	*/
-	syslog_write(U_INF, "reading data for '%s' from cfg-file",
-						ctx->username);
+	syslog_write(U_INF, "reading data for '%s' from cfg-file", who);
 
 	/*
 	** Evaluate the destination FTP server address.
@@ -192,30 +195,31 @@ void ldap_setup_user(CONTEXT *ctx)
 	*/
 	if (ctx->magic_addr != INADDR_ANY)
 		ctx->srv_addr = ctx->magic_addr;
-	else if ((ctx->srv_addr = config_addr(ctx->username,
+	else if ((ctx->srv_addr = config_addr(who,
 			"DestinationAddress",
 			INADDR_BROADCAST)) == INADDR_BROADCAST) {
 		errno = 0;
 		syslog_error("can't eval DestAddr for %s",
-					ctx->cli_ctrl->peer);
-		exit(EXIT_FAILURE);
+		             ctx->cli_ctrl->peer);
+		return -1;
 	}
 #if defined(COMPILE_DEBUG)
 	debug(2, "DestAddr for %s: '%s'", ctx->cli_ctrl->peer,
-				socket_addr2str(ctx->srv_addr));
+	         socket_addr2str(ctx->srv_addr));
 #endif
 
 	/*
 	** Evaluate the destination FTP server port
 	*/
-	if (ctx->magic_port != (u_int16_t) 0)
+	if (ctx->magic_port != (u_int16_t)INPORT_ANY)
 		ctx->srv_port = ctx->magic_port;
-	else if ((ctx->srv_port = config_port(ctx->username,
-			"DestinationPort", IPPORT_FTP)) == 0) {
+	else if ((ctx->srv_port = config_port(who,
+		 "DestinationPort", IPPORT_FTP)) == 0)
+	{
 		errno = 0;
 		syslog_error("can't eval DestPort for %s",
 					ctx->cli_ctrl->peer);
-		exit(EXIT_FAILURE);
+		return -1;
 	}
 #if defined(COMPILE_DEBUG)
 	debug(2, "DestPort for %s: %d",
@@ -225,8 +229,7 @@ void ldap_setup_user(CONTEXT *ctx)
 	/*
 	** Evaluate the destination transfer mode
 	*/
-	p = config_str(ctx->username,
-			"DestinationTransferMode", "client");
+	p = config_str(who, "DestinationTransferMode", "client");
 	if (strcasecmp(p, "active") == 0)
 		ctx->srv_mode = MOD_ACT_FTP;
 	else if (strcasecmp(p, "passive") == 0)
@@ -236,7 +239,7 @@ void ldap_setup_user(CONTEXT *ctx)
 	else {
 		syslog_error("can't eval DestMode for %s",
 					ctx->cli_ctrl->peer);
-		exit(EXIT_FAILURE);
+		return -1;
 	}
 #if defined(COMPILE_DEBUG)
 	debug(2, "DestMode for %s: %s", ctx->cli_ctrl->peer, p);
@@ -245,8 +248,8 @@ void ldap_setup_user(CONTEXT *ctx)
 	/*
 	** Evaluate the port ranges
 	*/
-	l = config_port(ctx->username, "DestinationMinPort", INPORT_ANY);
-	u = config_port(ctx->username, "DestinationMaxPort", INPORT_ANY);
+	l = config_port(who, "DestinationMinPort", INPORT_ANY);
+	u = config_port(who, "DestinationMaxPort", INPORT_ANY);
 	if (l > 0 && u > 0 && u >= l) {
 		ctx->srv_lrng = l;
 		ctx->srv_urng = u;
@@ -259,8 +262,8 @@ void ldap_setup_user(CONTEXT *ctx)
 			(int) ctx->srv_lrng, (int) ctx->srv_urng);
 #endif
 
-	l = config_port(ctx->username, "ActiveMinDataPort", INPORT_ANY);
-	u = config_port(ctx->username, "ActiveMaxDataPort", INPORT_ANY);
+	l = config_port(who, "ActiveMinDataPort", INPORT_ANY);
+	u = config_port(who, "ActiveMaxDataPort", INPORT_ANY);
 	if (l > 0 && u > 0 && u >= l) {
 		ctx->act_lrng = l;
 		ctx->act_urng = u;
@@ -279,8 +282,8 @@ void ldap_setup_user(CONTEXT *ctx)
 			(int) ctx->act_lrng, (int) ctx->act_urng);
 #endif
 
-	l = config_port(ctx->username, "PassiveMinDataPort", INPORT_ANY);
-	u = config_port(ctx->username, "PassiveMaxDataPort", INPORT_ANY);
+	l = config_port(who, "PassiveMinDataPort", INPORT_ANY);
+	u = config_port(who, "PassiveMaxDataPort", INPORT_ANY);
 	if (l > 0 && u > 0 && u >= l) {
 		ctx->pas_lrng = l;
 		ctx->pas_urng = u;
@@ -296,8 +299,8 @@ void ldap_setup_user(CONTEXT *ctx)
 	/*
 	** Setup other configuration options
 	*/
-	ctx->same_adr = config_bool(ctx->username, "SameAddress", 1);
-	ctx->timeout  = config_int (ctx->username, "TimeOut",   900);
+	ctx->same_adr = config_bool(who, "SameAddress", 1);
+	ctx->timeout  = config_int (who, "TimeOut",   900);
 #if defined(COMPILE_DEBUG)
 	debug(2, "SameAddress for %s: %s", ctx->cli_ctrl->peer,
 				ctx->same_adr ? "yes" : "no");
@@ -308,8 +311,10 @@ void ldap_setup_user(CONTEXT *ctx)
 	/*
 	** Adjust the allow/deny flags for the commands
 	*/
-	p = config_str(ctx->username, "ValidCommands", NULL);
+	p = config_str(who, "ValidCommands", NULL);
 	cmds_set_allow(p);
+
+	return 0;
 }
 
 
@@ -319,29 +324,72 @@ void ldap_setup_user(CONTEXT *ctx)
 **	Function......:	ldap_fetch
 **
 **	Parameters....:	ctx		Pointer to user context
+**			srv		Pointer ldap server name
+**			who		Pointer to user/auth name
+**			pwd		Pointer to user/auth pwd
 **
-**	Return........:	LDAP_SUCCESS or LDAP_... error number
+**	Return........:	0 on success
 **
 **	Purpose.......: Read the user specific parameters from
-**			an LDAP Server. The return values is
-**			only used for error handling, and not
-**			to determine if the user is known to
-**			the server. This is actually handled
-**			by setting the destination address.
+**			an LDAP Server.
 **
 ** ------------------------------------------------------------ */
 
-static int ldap_fetch(CONTEXT *ctx, char *srv)
+static int ldap_fetch(CONTEXT *ctx, char *srv, char *who, char *pwd)
 {
-	char str[MAX_PATH_SIZE], *p, *base, *idnt, *objc;
-	char *bind_dn, *bind_pw;
-	int port, lderr;
+	char str[MAX_PATH_SIZE], *ptr, *p;
+	char *bind_dn, *bind_pw, *base, *idnt, *objc;
+	int port, lderr, is_fmt;
 	LDAP *ld;
 	LDAPMessage *result, *e;
 	u_int16_t l, u;
 
-	if (ctx == NULL || srv == NULL)		/* Basic sanity */
-		misc_die(FL, "ldap_fetch: ?ctx? ?srv?");
+	/* Basic sanity */
+	if (ctx == NULL || srv == NULL || who == NULL)
+		misc_die(FL, "ldap_fetch: ?ctx? ?srv? ?who?");
+
+	/*
+	** prepare bind_dn: LDAPBindDN may contain
+	** _one_ %s (printf-fmt); replace it with
+	** the name passed via who parmeter...
+	*/
+	is_fmt  = 0;
+	bind_dn = NULL;
+	bind_pw = NULL;
+	if( (ptr = config_str(NULL, "LDAPBindDN", NULL))) {
+		size_t len;
+
+		/*
+		** search for exactly one %s; if any other
+		** fmt's are present report an parse error...
+		*/
+		for(p=ptr; p && p[0] && (p = strchr(p, '%')); p++) {
+			/* ignore %% pairs...? */
+			if('%' == p[1]) continue;
+			if(!is_fmt && ('s' == p[1] || 'S' == p[1])) {
+				is_fmt = 1;
+			} else {
+				errno = 0;
+				misc_die(FL, "ldap_fetch: ?BindDN-Fmt?");
+				return -1;
+			}
+		}
+
+		if( is_fmt) {
+			/*
+			** LDAPBindDN is a fmt-string
+			*/
+			len = strlen(who) + strlen(ptr);
+			bind_dn = misc_alloc(FL, len);
+#if defined(HAVE_SNPRINTF)
+			snprintf(bind_dn, len, ptr, who);
+#else
+			sprintf(bind_dn, ptr, who);
+#endif
+		} else {
+			bind_dn = misc_strdup(FL, ptr);
+		}
+	}
 
 	/*
 	** Determine LDAP server and port
@@ -350,8 +398,9 @@ static int ldap_fetch(CONTEXT *ctx, char *srv)
 	if ((p = strchr(str, ':')) != NULL) {
 		*p++ = '\0';
 		port = (int) socket_str2port(p, LDAP_PORT);
-	} else
+	} else {
 		port = (int) LDAP_PORT;
+	}
 
 #if defined(COMPILE_DEBUG)
 	debug(2, "LDAP server: %s:%d", str, port);
@@ -361,26 +410,53 @@ static int ldap_fetch(CONTEXT *ctx, char *srv)
 	** Ready to contact the LDAP server (as anonymous user)
 	*/
 	if ((ld = ldap_init(str, port)) == NULL) {
-		syslog_error("can't reach LDAP server %s:%d",
-						str, port);
-		exit(EXIT_FAILURE);
+		errno = 0;
+		syslog_error("can't reach LDAP server %s:%d", str, port);
+		if(NULL != bind_dn)
+			misc_free(FL, bind_dn);
+		return -1;
+	} else {
+		syslog_write(T_DBG, "LDAP server %s:%d: init succeed",
+		                    str, port);
 	}
 
-	bind_pw = NULL;
-	if( (bind_dn = config_str(NULL, "LDAPBindDN", NULL))) {
-		bind_pw = config_str(NULL, "LDAPBindPW", "");
+	if((NULL != bind_dn)) {
+		if( !is_fmt) {
+			/*
+			** use LDAPBindPW for bind if not a fmt bind_dn
+			*/
+			bind_pw = config_str(NULL, "LDAPBindPW", NULL);
+		} else	bind_pw = pwd;
 	}
 
-	if(LDAP_SUCCESS != (lderr = ldap_simple_bind_s(ld,
-		                    bind_dn, bind_pw)))
-		return lderr;
-
 	/*
-	** Search for the user
+	** now, bind as anonymous or authenticated
+	*/
+	if(LDAP_SUCCESS != (lderr = ldap_simple_bind_s(ld, bind_dn, bind_pw)))
+	{
+		ldap_unbind(ld); /* needed? */
+		errno = 0;
+		if(NULL != bind_dn) {
+			syslog_error("can't bind LDAP using dn '%s'", bind_dn);
+			misc_free(FL, bind_dn);
+		} else {
+			syslog_error("can't bind LDAP anonymously");
+		}
+		return -1;
+	} else {
+		syslog_write(T_DBG,
+		             "LDAP bind dn='%.256s' pw='%.256s': succeed",
+		             NIL(bind_dn), NIL(bind_pw));
+	}
+	if(NULL != bind_dn) {
+		misc_free(FL, bind_dn);
+		bind_dn = 0;
+	}
+
+	/*
+	** construct filter for the search
 	**	(by LDAPIdentifier, maybe also ObjectClass)
 	*/
-	if ((base = config_str(NULL, "LDAPBaseDN", "")) == NULL)
-		misc_die(FL, "ldap_fetch: ?BaseDN?");
 	if ((objc = config_str(NULL, "LDAPObjectClass", "")) == NULL)
 		misc_die(FL, "ldap_fetch: ?ObjectClass?");
 	if ((idnt = config_str(NULL, "LDAPIdentifier", "")) == NULL)
@@ -390,40 +466,127 @@ static int ldap_fetch(CONTEXT *ctx, char *srv)
 	if (*objc == '\0') {
 #if defined(HAVE_SNPRINTF)
 		snprintf(str, sizeof(str), "(%.256s=%.256s)",
-			  idnt, ctx->username);
+			  idnt, who);
 #else
-		sprintf(str, "(%.256s=%.256s)", idnt, ctx->username);
+		sprintf(str, "(%.256s=%.256s)", idnt, who);
 #endif
 	} else {
 #if defined(HAVE_SNPRINTF)
 		snprintf(str, sizeof(str),
 			  "(&(ObjectClass=%.256s)(%.256s=%.256s))",
-			  objc,  idnt, ctx->username);
+			  objc,  idnt, who);
 #else
 		sprintf(str,
 			  "(&(ObjectClass=%.256s)(%.256s=%.256s))",
-			  objc,  idnt, ctx->username);
+			  objc,  idnt, who);
 #endif
 	}
 
-#if defined(COMPILE_DEBUG)
-	debug(2, "LDAP search: base='%.256s' filter='%s'", base, str);
-#endif
 
-	if ((lderr = ldap_search_s(ld, base, LDAP_SCOPE_SUBTREE,
-			str, NULL, 0, &result)) != LDAP_SUCCESS) {
-		ldap_unbind(ld);
-		return lderr;
+	syslog_write(U_INF, "reading data for '%s' from LDAP", who);
+	if ((base = config_str(NULL, "LDAPBaseDN", NULL)) != NULL)
+	{
+		syslog_write(T_DBG,
+		             "LDAP search: base='%.256s' filter='%.256s'",
+		             base, str);
+
+		if(LDAP_SUCCESS != (lderr = ldap_search_s(ld, base,
+	           LDAP_SCOPE_SUBTREE, str, NULL, 0, &result)))
+		{
+			errno = 0;
+			syslog_error("can't read LDAP data for %s: %.512s",
+			             ctx->cli_ctrl->peer,
+			             ldap_err2string(lderr));
+			ldap_unbind(ld);
+			return -1;
+		}
+
+		/*
+		** Check if we have a user data
+		** (else return 'error' or 'empty')
+		*/
+		if ((e = ldap_first_entry(ld, result)) == NULL)
+		{
+			GET_LDERROR(ld,lderr);
+			errno = 0;
+			syslog_error("can't read LDAP data for %s: %.512s",
+			             ctx->cli_ctrl->peer,
+			             ldap_err2string(lderr));
+			ldap_msgfree(result);
+			ldap_unbind(ld);
+			return -1;
+		}
+	} else {
+		e = result = 0;
 	}
 
 	/*
-	** Check if we have a user (else return 'error' or 'empty')
+	** Preform auth on userauth if type is ldap
 	*/
-	if ((e = ldap_first_entry(ld, result)) == NULL) {
-		GET_LDERROR(ld,lderr);
-		ldap_msgfree(result);
+	if( (ptr = config_str(NULL, "UserAuthType", NULL))
+	    && !strcasecmp(ptr, "ldap"))
+	{
+		LDAPMessage *res=0, *a=e;
+		/*
+		** if LDAPAuthDN set, do auth on a different base...
+		*/
+		if(NULL != (ptr = config_str(NULL, "LDAPAuthDN", NULL)))
+		{
+			syslog_write(T_DBG,
+			       "LDAP auth: base='%.256s' filter='%.256s'",
+			       ptr, str);
+
+			if(LDAP_SUCCESS != (lderr = ldap_search_s(ld,
+			   ptr, LDAP_SCOPE_SUBTREE, str, NULL, 0, &res)))
+			{
+				syslog_error("can't read LDAP auth-data"
+					     " for %s: %.512s",
+					     ctx->cli_ctrl->peer,
+					     ldap_err2string(lderr));
+				if(result) ldap_msgfree(result);
+				ldap_unbind(ld);
+				return -1;
+			}
+
+			if(NULL == (a = ldap_first_entry(ld, res))) {
+				GET_LDERROR(ld,lderr);
+				errno = 0;
+				syslog_error("can't read LDAP auth-data"
+					     " for %s: %.512s",
+					     ctx->cli_ctrl->peer,
+					     ldap_err2string(lderr));
+				if(result) ldap_msgfree(result);
+				ldap_msgfree(res);
+				ldap_unbind(ld);
+				return -1;
+			}
+		} else if(NULL == base || NULL == e) {
+			ldap_unbind(ld);
+			misc_die(FL, "ldap_fetch: ?LDAPBaseDN?");
+		}
+
+		/*
+		** OK, let's check the user auth now
+		*/
+		if(0 != ldap_auth(ld, a, who, pwd)) {
+			errno = 0;
+			syslog_write(U_ERR,
+			             "LDAP user auth failed for %s from %s",
+			             who, ctx->cli_ctrl->peer);
+			if(result) ldap_msgfree(result);
+			if(res)    ldap_msgfree(res);
+			ldap_unbind(ld);
+			return -1;
+		}
+		if(res) ldap_msgfree(res);
+	}
+
+	/*
+	** read proxy user profile data if we have a base
+	*/
+	if(NULL == base) {
 		ldap_unbind(ld);
-		return lderr;
+		return 0;
 	}
 
 	/*
@@ -431,14 +594,17 @@ static int ldap_fetch(CONTEXT *ctx, char *srv)
 	** This is mandatory! Refuse to run if none given.
 	*/
 	p = ldap_attrib(ld, e, "DestinationAddress", "255.255.255.255");
-	if (ctx->magic_addr != INADDR_ANY)
+	if (ctx->magic_addr != INADDR_ANY) {
 		ctx->srv_addr = ctx->magic_addr;
-	else if ((ctx->srv_addr = socket_str2addr(p,
+	} else
+	if ((ctx->srv_addr = socket_str2addr(p,
 			INADDR_BROADCAST)) == INADDR_BROADCAST) {
 		errno = 0;
 		syslog_error("can't eval DestAddr for %s",
 					ctx->cli_ctrl->peer);
-		exit(EXIT_FAILURE);
+		ldap_msgfree(result);
+		ldap_unbind(ld);
+		return -1;
 	}
 #if defined(COMPILE_DEBUG)
 	debug(2, "DestAddr for %s: '%s'", ctx->cli_ctrl->peer,
@@ -449,14 +615,17 @@ static int ldap_fetch(CONTEXT *ctx, char *srv)
 	** Evaluate the destination FTP server port
 	*/
 	p = ldap_attrib(ld, e, "DestinationPort", "ftp");
-	if (ctx->magic_port != (u_int16_t) 0)
+	if (ctx->magic_port != (u_int16_t)INPORT_ANY) {
 		ctx->srv_port = ctx->magic_port;
-	else if ((ctx->srv_port = socket_str2port(p,
-				IPPORT_FTP)) == (u_int16_t) 0) {
+	} else
+	if ((ctx->srv_port = socket_str2port(p,
+			IPPORT_FTP)) == (u_int16_t)INPORT_ANY) {
 		errno = 0;
 		syslog_error("can't eval DestPort for %s",
 					ctx->cli_ctrl->peer);
-		exit(EXIT_FAILURE);
+		ldap_msgfree(result);
+		ldap_unbind(ld);
+		return -1;
 	}
 #if defined(COMPILE_DEBUG)
 	debug(2, "DestPort for %s: %d",
@@ -474,9 +643,12 @@ static int ldap_fetch(CONTEXT *ctx, char *srv)
 	else if (strcasecmp(p, "client") == 0)
 		ctx->srv_mode = MOD_CLI_FTP;
 	else {
+		errno = 0;
 		syslog_error("can't eval DestMode for %s",
 					ctx->cli_ctrl->peer);
-		exit(EXIT_FAILURE);
+		ldap_msgfree(result);
+		ldap_unbind(ld);
+		return -1;
 	}
 #if defined(COMPILE_DEBUG)
 	debug(2, "DestMode for %s: %s", ctx->cli_ctrl->peer, p);
@@ -578,7 +750,145 @@ static int ldap_fetch(CONTEXT *ctx, char *srv)
 	*/
 	ldap_msgfree(result);
 	ldap_unbind(ld);
-	return LDAP_SUCCESS;
+
+	return 0;
+}
+
+
+/* ------------------------------------------------------------ **
+**
+**	Function......:	ldap_auth
+**
+**	Parameters....:	ld		Pointer to LDAP struct
+**			e		Pointer to result buffer
+**			who		Pointer to user name
+**			pwd		Pointer to user pwd
+**
+**	Return........:	0 on success
+**
+**	Purpose.......: Preform LDAP userauth
+**
+** ------------------------------------------------------------ */
+
+static int   ldap_auth(LDAP *ld, LDAPMessage *e, char *who, char *pwd)
+{
+	char str[MAX_PATH_SIZE];
+	char *v, *p, *q;
+	size_t len;
+
+	if (ld == NULL || e == NULL)
+		misc_die(FL, "ldap_checkauth: ?ld? ?e?");
+
+	/*
+	** check "user enabled" flag if present
+	*/
+	if( (p = config_str(NULL, "LDAPAuthOKFlag", NULL))) {
+		misc_strncpy(str, p, sizeof(str));
+		if( (v = strchr(str, '=')))
+			*v++ = '\0';
+		else	v = 0;
+
+		if(v && strlen(v) && strlen(str)) {
+			if(0 != ldap_exists(ld, e, str, v, 0)) {
+				syslog_write(U_WRN,
+				"access denied for %s", NIL(who));
+				return -1;
+			} else {
+				syslog_write(T_DBG,
+				"LDAP auth ok-check: '%.256s'='%.256s' passed",
+				NIL(str), NIL(v));
+			}
+		} else {
+			errno = 0;
+			misc_die(FL, "ldap_auth: ?LDAPAuthOKFlag?");
+		}
+	} else {
+		syslog_write(T_DBG, "LDAP auth ok-check skipped");
+	}
+
+	/*
+	** check user pass match
+	*/
+	if( (p = config_str(NULL, "LDAPAuthPWAttr", "")) && strlen(p)) {
+		if(NULL == pwd)
+			pwd = "";
+
+		v   = config_str(NULL, "LDAPAuthPWType", "plain");
+		q   = ldap_attrib(ld, e, p, "");
+		p   = 0;
+		len = 0;
+
+		if( !strncasecmp(v, "plain", sizeof("plain")-1)) {
+			/*
+			** plain passwd - no prefix
+			*/
+			len = sizeof("plain")-1;
+			p   = pwd;
+#if defined(HAVE_CRYPT)
+		} else
+		if( !strncasecmp(v, "crypt", sizeof("plain")-1)) {
+			/*
+			** crypt - no prefix
+			*/
+			len = sizeof("plain")-1;
+			p   = crypt(pwd, q);
+		} else
+		if( !strncasecmp(v, "{crypt}", sizeof("{crypt}")-1)) {
+			/*
+			** crypt - {crypt} prefix
+			*/
+			len = sizeof("{crypt}")-1;
+			if(strncasecmp(q, "{crypt}", len)) {
+				syslog_error("ldap user auth - prefix missed");
+				return -1;
+			}
+			q+=len;
+			p = crypt(pwd, q);
+#endif
+		} else {
+			errno = 0;
+			misc_die(FL, "ldap_auth: ?LDAPAuthPWType?");
+		}
+
+		/*
+		** check if we have different minimal length
+		** it is coded in latest "char", i.e. plain9
+		*/
+		if(0 < len && strlen(v) == len+1 &&
+		   '0' <= v[len] && '9' >= v[len])
+		{
+			len = (size_t)v[len] - '0';
+		} else	len = PASS_MIN_LEN;
+
+		syslog_write(T_DBG, "LDAP auth pw-type[%d]='%.256s'", len, v);
+#if defined(COMPILE_DEBUG)
+		debug(3,            "LDAP auth pw-check: '%.256s' ?= '%.256s'",
+		                    NIL(q), NIL(p));
+#endif
+
+		/*
+		** check (lenght) and compare passwds; the user
+		** account is locked if LDAP-PWD is "*" or "!"
+		*/
+		if(p && strlen(p)>=len && strlen(q) == strlen(p) &&
+		   !(1==strlen(q) && ('*' == q[0] || '!' == q[0])))
+		{
+			if(0 == strcmp(q, p)) {
+				syslog_write(T_DBG,
+				             "LDAP auth pw-check succeed");
+				return 0;
+			}
+		}
+		syslog_write(T_DBG, "LDAP auth pw-check failed");
+		return -1;
+	} else {
+		syslog_write(T_DBG, "LDAP auth pw-check skipped");
+	}
+
+	/*
+	** OK, all configured LDAPAuth stuff succeed...
+	*/
+	return 0;
 }
 
 
@@ -630,11 +940,86 @@ static char *ldap_attrib(LDAP *ld, LDAPMessage *e, char *attr, char *dflt)
 #endif
 	return str;
 }
+
+
+/* ------------------------------------------------------------ **
+**
+**	Function......:	ldap_exists
+**
+**	Parameters....:	ld		Pointer to LDAP struct
+**			e		Pointer to result buffer
+**			attr		Name of desired option
+**			vstr		Value of desired option
+**			cs		1 for case sensitive check
+**
+**	Return........:	0 if value found
+**
+**	Purpose.......: Search the LDAP result message for the
+**			desired (multivalue) attribute and check
+**			if it contains a value string.
+**
+** ------------------------------------------------------------ */
+
+static int  ldap_exists(LDAP *ld, LDAPMessage *e, char *attr,
+                                                  char *vstr, int cs)
+{
+	char **vals;
+	int    count, at;
+
+	if (ld == NULL || e == NULL || attr == NULL || vstr == NULL)
+		misc_die(FL, "ldap_exists: ?ld? ?e? ?attr? ?vstr?");
+
+	/*
+	** See if this attribute has values available
+	*/
+	if ((vals = ldap_get_values(ld, e, attr)) == NULL) {
+#if defined(COMPILE_DEBUG)
+		debug(3, "LDAP result: no values for '%.256s'", attr);
+#endif
+		return -1;
+	}
+
+	count = ldap_count_values(vals);
+	if(cs) {
+		for(at=0; at < count; at++) {
+#if defined(COMPILE_DEBUG)
+		debug(3, "LDAP result: checking[%d:%d] '%.256s'='%.1024s'",
+		                              count-1, at, attr, vals[at]);
+#endif
+			if(misc_strequ(vals[at], vstr)) {
+				ldap_value_free(vals);
+				return 0;
+			}
+		}
+	} else {
+		for(at=0; at < count; at++) {
+#if defined(COMPILE_DEBUG)
+		debug(3, "LDAP result: checking[%d:%d] '%.256s'='%.1024s'",
+		                              count-1, at, attr, vals[at]);
+#endif
+			if(misc_strcaseequ(vals[at], vstr)) {
+				ldap_value_free(vals);
+				return 0;
+			}
+		}
+	}
+	ldap_value_free(vals);
+
+#if defined(COMPILE_DEBUG)
+		debug(3, "LDAP result: '%.256s'='%.1024s' not found",
+		                       attr, vstr);
+#endif
+
+	return 1;
+}
 #endif
 
 
 /* ------------------------------------------------------------
  * $Log: ftp-ldap.c,v $
+ * Revision 1.7  2002/05/02 13:17:12  mt
+ * implemented simple (ldap based) user auth
+ *
  * Revision 1.6  2002/01/14 19:26:38  mt
  * implemented bind_dn and pwd authorized ldap_simple_bind
  * fixed ld_errno fetching macro to work with openldap 2.x
